@@ -5,6 +5,7 @@ import os
 import platform
 import itertools
 import distutils.errors
+
 from setuptools.extern.six.moves import filterfalse
 
 if platform.system() == 'Windows':
@@ -84,6 +85,13 @@ def patch_for_specialized_compiler():
     except Exception:
         pass
 
+    try:
+        # Patch distutils._msvccompiler.MSVCCompiler.library_dir_option
+        unpatched['msvc14_library_dir_option'] = msvc14compiler.MSVCCompiler.library_dir_option
+        msvc14compiler.MSVCCompiler.library_dir_option = msvc14_library_dir_option
+    except Exception:
+        pass
+
 
 def msvc9_find_vcvarsall(version):
     """
@@ -211,6 +219,27 @@ def msvc14_get_vc_env(plat_spec):
         raise
 
 
+def msvc14_library_dir_option(self, dir):
+    """
+    Patched "distutils._msvccompiler.MSVCCompiler.library_dir_option"
+    to fix unquoted path in "\LIBPATH" argument when a space is on path.
+
+    Parameters
+    ----------
+    dir: str
+        Path to convert in "\LIBPATH" argument.
+
+    Return
+    ------
+    "\LIBPATH" argument: str
+    """
+    opt = unpatched['msvc14_library_dir_option'](self, dir)
+    if ' ' in opt and '"' not in opt:
+        # Quote if space and not already quoted
+        opt = '"%s"' % opt
+    return opt
+
+
 def _augment_exception(exc, version, arch=''):
     """
     Add details to the exception message to help guide the user
@@ -221,7 +250,7 @@ def _augment_exception(exc, version, arch=''):
 
     if "vcvarsall" in message.lower() or "visual c" in message.lower():
         # Special error message if MSVC++ not installed
-        tmpl = 'Microsoft Visual C++ {version:0.1f} is required {message}.'
+        tmpl = 'Microsoft Visual C++ {version:0.1f} is required.'
         message = tmpl.format(**locals())
         msdownload = 'www.microsoft.com/download/details.aspx?id=%d'
         if version == 9.0:
@@ -239,6 +268,10 @@ def _augment_exception(exc, version, arch=''):
             # For VC++ 10.0 Redirect user to Windows SDK 7.1
             message += ' Get it with "Microsoft Windows SDK 7.1": '
             message += msdownload % 8279
+        elif version >= 14.0:
+            # For VC++ 14.0 Redirect user to Visual C++ Build Tools
+            message += (' Get it with "Microsoft Visual C++ Build Tools": '
+                r'http://landinghub.visualstudio.com/visual-cpp-build-tools')
 
     exc.args = (message, )
 
@@ -351,23 +384,12 @@ class RegistryInfo:
     def __init__(self, platform_info):
         self.pi = platform_info
 
-    @property
-    def microsoft(self):
-        """
-        Microsoft software registry key.
-        """
-        return os.path.join(
-            'Software',
-            '' if self.pi.current_is_x86() else 'Wow6432Node',
-            'Microsoft',
-        )
-
     @property
     def visualstudio(self):
         """
         Microsoft Visual Studio root registry key.
         """
-        return os.path.join(self.microsoft, 'VisualStudio')
+        return 'VisualStudio'
 
     @property
     def sxs(self):
@@ -395,15 +417,14 @@ class RegistryInfo:
         """
         Microsoft Visual C++ for Python registry key.
         """
-        path = r'DevDiv\VCForPython'
-        return os.path.join(self.microsoft, path)
+        return r'DevDiv\VCForPython'
 
     @property
     def microsoft_sdk(self):
         """
         Microsoft SDK registry key.
         """
-        return os.path.join(self.microsoft, 'Microsoft SDKs')
+        return 'Microsoft SDKs'
 
     @property
     def windows_sdk(self):
@@ -424,11 +445,29 @@ class RegistryInfo:
         """
         Microsoft Windows Kits Roots registry key.
         """
-        return os.path.join(self.microsoft, r'Windows Kits\Installed Roots')
+        return r'Windows Kits\Installed Roots'
+
+    def microsoft(self, key, x86=False):
+        """
+        Return key in Microsoft software registry.
+
+        Parameters
+        ----------
+        key: str
+            Registry key path where look.
+        x86: str
+            Force x86 software registry.
+
+        Return
+        ------
+        str: value
+        """
+        node64 = '' if self.pi.current_is_x86() or x86 else r'\Wow6432Node'
+        return os.path.join('Software', node64, 'Microsoft', key)
 
     def lookup(self, key, name):
         """
-        Look for values in registry.
+        Look for values in registry in Microsoft software registry.
 
         Parameters
         ----------
@@ -441,14 +480,23 @@ class RegistryInfo:
         ------
         str: value
         """
+        KEY_READ = winreg.KEY_READ
+        openkey = winreg.OpenKey
+        ms = self.microsoft
         for hkey in self.HKEYS:
             try:
-                bkey = winreg.OpenKey(hkey, key, 0, winreg.KEY_READ)
-            except IOError:
-                continue
+                bkey = openkey(hkey, ms(key), 0, KEY_READ)
+            except (OSError, IOError):
+                if not self.pi.current_is_x86():
+                    try:
+                        bkey = openkey(hkey, ms(key, True), 0, KEY_READ)
+                    except (OSError, IOError):
+                        continue
+                else:
+                    continue
             try:
                 return winreg.QueryValueEx(bkey, name)[0]
-            except IOError:
+            except (OSError, IOError):
                 pass
 
 
@@ -491,7 +539,7 @@ class SystemInfo:
             for key in vckeys:
                 try:
                     bkey = winreg.OpenKey(hkey, key, 0, winreg.KEY_READ)
-                except IOError:
+                except (OSError, IOError):
                     continue
                 subkeys, values, _ = winreg.QueryInfoKey(bkey)
                 for i in range(values):
@@ -765,6 +813,7 @@ class EnvironmentInfo:
     """
     # Variables and properties in this class use originals CamelCase variables
     # names from Microsoft source files for more easy comparaison.
+
     def __init__(self, arch, vc_ver=None, vc_min_ver=None):
         self.pi = PlatformInfo(arch)
         self.ri = RegistryInfo(self.pi)
@@ -803,7 +852,7 @@ class EnvironmentInfo:
         Microsoft Visual C++ & Microsoft Foundation Class Includes
         """
         return [os.path.join(self.si.VCInstallDir, 'Include'),
-                os.path.join(self.si.VCInstallDir, 'ATLMFC\Include')]
+                os.path.join(self.si.VCInstallDir, r'ATLMFC\Include')]
 
     @property
     def VCLibraries(self):
@@ -903,7 +952,7 @@ class EnvironmentInfo:
                 os.path.join(self.si.WindowsSdkDir, 'UnionMetadata'),
                 os.path.join(
                     ref,
-                    'Windows.Foundation.UniversalApiContract'
+                    'Windows.Foundation.UniversalApiContract',
                     '1.0.0.0',
                 ),
                 os.path.join(
@@ -913,7 +962,7 @@ class EnvironmentInfo:
                 ),
                 os.path.join(
                     ref,
-                    'Windows.Networking.Connectivity.WwanContract'
+                    'Windows.Networking.Connectivity.WwanContract',
                     '1.0.0.0',
                 ),
                 os.path.join(
@@ -1189,5 +1238,5 @@ class EnvironmentInfo:
             if name:
                 return '%s\\' % name[0]
             return ''
-        except IOError:
+        except (OSError, IOError):
             return ''
